﻿#pragma once
#include <Windows.h>
#include <map>
// создание класса окна
class Window
{
private:
	typedef LRESULT(Window::* FuncPointer)(LPARAM, WPARAM);
	struct POINTER
	{
		Window* wnd;
		FuncPointer func;
	};

protected:
	std::map<UINT, POINTER> m_msgMap;

public:
	HINSTANCE m_hInstance;
	HWND m_hWnd;
	int m_Width, m_Height;

	void Size(
		int X,
		int Y,
		int nWidth,
		int nHeight
	) {
		MoveWindow
		(
			m_hWnd,
			X,
			Y,
			nWidth,
			nHeight,
			true
		);
	}

	bool RegisterWndClass() {
		WNDCLASSEXW wcex;
		wcex.lpszClassName = L"Window";
		wcex.cbSize = sizeof(WNDCLASSEX);
		wcex.cbClsExtra = 0;
		wcex.cbWndExtra = 0;
		wcex.lpfnWndProc = WNDPROC(_WndProc);
		wcex.hbrBackground = (HBRUSH)(COLOR_WINDOW);
		wcex.hInstance = GetModuleHandle(0);
		wcex.style = CS_HREDRAW | CS_VREDRAW;
		wcex.hCursor = LoadCursor(nullptr, IDC_ARROW);
		wcex.hIcon = 0;
		wcex.hIconSm = 0;
		wcex.lpszMenuName = 0;

		if (!RegisterClassExW(&wcex))
			return false;
		return true;
	}
	// координаты окна
	BOOL Create(
		DWORD dwExStyle,
		LPCWSTR lpClassName,
		LPCWSTR lpWindowName,
		DWORD dwStyle,
		int X,
		int Y,
		int nWidth,
		int nHeight,
		HWND hWndParent,
		HMENU hMenu,
		HINSTANCE hInstance,
		LPVOID lpParam
	) {
		m_hWnd = CreateWindowEx
		(
			dwExStyle,
			lpClassName,
			lpWindowName,
			dwStyle | WS_CLIPCHILDREN,
			X,
			Y,
			nWidth,
			nHeight,
			hWndParent,
			hMenu,
			hInstance,
			lpParam
		);

		if (!m_hWnd)
			return false;

		return true;
	}
	// получение сообщений, которая ОС Windows будет отсылать окну
	static LRESULT CALLBACK _WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
		Window* wnd = NULL;
		if (message == WM_NCCREATE) {
			wnd = (Window*)LPCREATESTRUCT(lParam)->lpCreateParams;
			SetWindowLong(hWnd, GWLP_USERDATA, LONG(LPCREATESTRUCT(lParam)->lpCreateParams));
			wnd->m_hWnd = hWnd;
		}

		wnd = (Window*)GetWindowLong(hWnd, GWLP_USERDATA);
		if (wnd) {
			std::map<UINT, POINTER>::iterator it;
			it = wnd->m_msgMap.find(message);

			if (it != wnd->m_msgMap.end())
			{
				POINTER msg = it->second;
				LRESULT result = (msg.wnd->*msg.func)(lParam, wParam);
				if (result)
					return result;
			}
		}
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	// добавление сообщений
	template<typename T>
	bool AddMessage(UINT message, Window* wnd, LRESULT(T::* funcpointer)(LPARAM, WPARAM)) {
		if (!wnd || !funcpointer)
			return false;

		POINTER msg;
		msg.wnd = wnd;
		msg.func = reinterpret_cast<FuncPointer>(funcpointer);

		m_msgMap.insert(std::pair<UINT, POINTER>(message, msg));
		return true;
	}
};