﻿#include <Windows.h>
#include <commdlg.h>
#include <WinError.h>
#include "resource.h"
#include "framework.h"

//"костяк" окна и его состав
class MyWindow : public Window
{
private:
	fileStruct* arrayOfFiles = new fileStruct[NUM_OF_FILES]; // массив файлов
	CButton* m_hBtnCompare = new CButton; // кнопка сравнения файлов
	CButton* m_hBtnComparePrevious = new CButton; // кнопка сравнения предыдущего отличия
	CButton* m_hBtnCompareNext = new CButton; // кнопка сравнения следующего отличия
	CButton* m_hBtnFont = new CButton; // кнопка выбора шрифта
	CTextPainter m_text; // прорисовка текста
	CHOOSEFONT cf = { 0 }; // выбор шрифта

	SYSTEM_INFO m_sinf; // информация о системе

	int m_countOfLines; // подсчёт линий
	int m_wheelDelta; // количество делений, на которое повернулось колёсико мыши, умноженное на константу WHEEL_DELTA

	double test; // тест
public:
	MyWindow() { // окно
		m_countOfLines = 0; // подсчёт линий
		m_wheelDelta = 0;  // количество делений, на которое повернулось колёсико мыши, умноженное на константу WHEEL_DELTA
		GetSystemInfo(&m_sinf);
		// события
		AddMessage(WM_CREATE,        this, &MyWindow::OnCreate);
		AddMessage(WM_KEYDOWN,       this, &MyWindow::OnKeyDown);
		AddMessage(WM_MOUSEWHEEL,    this, &MyWindow::OnMouseWheel);
		AddMessage(WM_VSCROLL,		 this, &MyWindow::OnVScroll);
		AddMessage(WM_PAINT,         this, &MyWindow::OnPaint);
		AddMessage(WM_SIZE,          this, &MyWindow::OnSize);
		AddMessage(WM_COMMAND,       this, &MyWindow::OnClick);
		AddMessage(WM_MOVE,          this, &MyWindow::OnMove);	
		AddMessage(WM_GETMINMAXINFO, this, &MyWindow::OnGetMinMax);	
		AddMessage(WM_DESTROY,       this, &MyWindow::OnDestroy);
	}

	#pragma region MESSAGES
	//Обработка сообщения WM_CREATE
	LRESULT OnCreate(LPARAM lParam, WPARAM wParam) { // появление окна
		
		//надпись "Сравнить файлы" на кнопке для сравнения
		if (!m_hBtnCompare->Create(L"Сравнить файлы", BTN_COMPARE_X, BTN_COMPARE_Y, BTN_COMPARE_W(m_Width), BTN_COMPARE_H, m_hWnd))
		{

			showError();
			return 1;
		}

		//надпись "Предыдущее сравнение" на кнопке для поблочного сравнения
		if (!m_hBtnComparePrevious->Create(L"Предыдущее сравнение", BTN_COMPAREPREVIOUS_X, BTN_COMPAREPREVIOUS_Y, BTN_COMPAREPREVIOUS_W(m_Width), BTN_COMPAREPREVIOUS_H, m_hWnd))
		{

			showError();
			return 1;
		}

		//надпись "Следующее сравнение" на кнопке для поблочного сравнения
		if (!m_hBtnCompareNext->Create(L"Следующее сравнение", BTN_COMPARENEXT_X, BTN_COMPARENEXT_Y, BTN_COMPARENEXT_W(m_Width), BTN_COMPARENEXT_H, m_hWnd))
		{

			showError();
			return 1;
		}

		//надпись "Шрифт" на кнопке для сравнения
		if (!m_hBtnFont->Create(L"Шрифт", BTN_FONT_X, BTN_FONT_Y, BTN_FONT_W(m_Width), BTN_FONT_H, m_hWnd))
		{

			showError();
			return 1;
		}

		for (int i = 0; i < NUM_OF_FILES; i++) {
			arrayOfFiles[i].fileOpen.setId(i);
			if (!arrayOfFiles[i].fileOpen.createBlock(m_hWnd))
			{
				showError();
				return 1;
			}	
		}

		return 0;
	}
	
	//Обработка сообщения WM_KEYDOWN
	LRESULT OnKeyDown(LPARAM lParam, WPARAM wParam) {
		switch (wParam)
		{ // коды виртуальных клавиш для их задействования в программе
		case VK_HOME: // Home
			SendMessage(m_hWnd, WM_VSCROLL, SB_TOP, 0L);
			break;
		case VK_END: // End
			SendMessage(m_hWnd, WM_VSCROLL, SB_BOTTOM, 0L);
			break;
		case VK_PRIOR: // PageUp
			SendMessage(m_hWnd, WM_VSCROLL, SB_PAGEUP, 0L);
			break;
		case VK_NEXT: //PageDown
			SendMessage(m_hWnd, WM_VSCROLL, SB_PAGEDOWN, 0L);
			break;
		case VK_UP: // Up (стрелка)
			SendMessage(m_hWnd, WM_VSCROLL, SB_LINEUP, 0L);
			break;
		case VK_DOWN: // Down (стрелка)
			SendMessage(m_hWnd, WM_VSCROLL, SB_LINEDOWN, 0L);
			break;
		}
		return 0;
	}

	//Обработка сообщения WM_MOUSEWHEEL
	LRESULT OnMouseWheel(LPARAM lParam, WPARAM wParam) { // вращение колёсика мыши
		m_wheelDelta += GET_WHEEL_DELTA_WPARAM(wParam);

		for (; m_wheelDelta > WHEEL_DELTA; m_wheelDelta -= WHEEL_DELTA) {
			SendMessage(m_hWnd, WM_VSCROLL, SB_LINEUP, 0L);
		}

		for (; m_wheelDelta < 0; m_wheelDelta += WHEEL_DELTA) {
			SendMessage(m_hWnd, WM_VSCROLL, SB_LINEDOWN, 0L);
		}
		return 0;
	}

	//Обработка сообщения WM_VSCROLL
	LRESULT OnVScroll(LPARAM lParam, WPARAM wParam) {
		m_countOfLines = 0;

		switch (LOWORD(wParam)) 
		{
		case SB_PAGEUP: // изменение позиции ползунка на одну страницу вверх
		{
			printf("SB_PAGEUP\n");
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.m_scrollbar->pageUp(m_Height);
			}
		}
		break;
		case SB_PAGEDOWN: // изменение позиции ползунка на одну страницу вниз 
		{
			printf("SB_PAGEDOWN\n");
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.m_scrollbar->pageDown(m_Height);
			}
		}
		break;
		case SB_LINEUP: // изменение позиции ползунка на одну позицию вверх
		{
			printf("SB_LINEUP\n");
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.m_scrollbar->lineUp();
			}
		}
		break;
		case SB_LINEDOWN: // изменение позиции ползунка на одну позицию вниз
		{
			printf("SB_LINEDOWN\n");
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.m_scrollbar->lineDown();
			}
		}
		break;
		case SB_TOP: // прокрутка к верхней части (к самому началу)
		{
			printf("SB_TOP\n");
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.m_scrollbar->top();
			}
		}
		break;
		case SB_BOTTOM: // прокрутка к нижней части (к самому концу)
		{
			printf("SB_BOTTOM\n");
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.m_scrollbar->bottom();
			}
		}
		break;
		case SB_THUMBPOSITION:	// генерация перемещения ползунка на новую позицию
		{
			printf("SB_THUMBPOSITION\n");
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.m_scrollbar->mouseDrag(HIWORD(wParam));
			}
			break;
		}
		case SB_THUMBTRACK: // генерация перемещения ползунка на новое место, при этом удерживается кнопка мыши; отслеживание позиции ползунка перед тем, как он будет отпущен
		{
			printf("SB_THUMBTRACK\n");
			//printf((CHAR*)HIWORD(wParam)); // если активировать эту строку, то ползунок не будет двигаться: при попытке его движения программа вылетает или зависает
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.m_scrollbar->mouseDrag(HIWORD(wParam));
			}			
		}
		break;
		default:
			printf("0x%x\n", (LOWORD(wParam)));
			break;
		}
		
		for (int i = 0; i < NUM_OF_FILES; i++) {
			arrayOfFiles[i].fileOpen.m_scrollbar->setPose();
		}

		m_text.m_toDraw = true;
		InvalidateRect(m_hWnd, NULL, true);
		return 0;
	}

	//Обработка сообщения WM_PAINT
	LRESULT OnPaint(LPARAM lParam, WPARAM wParam) {
		PAINTSTRUCT ps;
		HDC hdc = BeginPaint(m_hWnd, &ps);
		SetBkMode(hdc, TRANSPARENT);
		
		if (m_text.m_toDraw) {
			m_text.FindBlock(arrayOfFiles, m_sinf.dwAllocationGranularity);
			m_text.drawLines
			(
				arrayOfFiles,
				hdc,
				arrayOfFiles[0].fileOpen.m_scrollbar->getPose() * LINE,
				m_countOfLines,
				m_Width,
				m_Height
			);

			m_text.m_toDraw = false;
		
		}
		EndPaint(m_hWnd, &ps);
		return 0;
	}

	//Обработка сообщения WM_SIZE
	LRESULT OnSize(LPARAM lParam, WPARAM wParam) {

		m_Width = LOWORD(lParam);
		m_Height = HIWORD(lParam);

		for (int i = 0; i < NUM_OF_FILES; i++)
		{
			arrayOfFiles[i].fileOpen.sizeBlock(m_Width, m_Height);
		}
		//отображение кнопки выбора второго файла + вывод содержимого обоих файлов
		m_hBtnCompare->Size
		(
			BTN_COMPARE_X,
			BTN_COMPARE_Y,
			BTN_COMPARE_W(m_Width),
			BTN_COMPARE_H
		);
		//отображение кнопки предыдущего сравнения
		m_hBtnComparePrevious->Size
		(
			BTN_COMPAREPREVIOUS_X,
			BTN_COMPAREPREVIOUS_Y,
			BTN_COMPAREPREVIOUS_W(m_Width),
			BTN_COMPAREPREVIOUS_H
		);
		//отображение кнопки следующего сравнения
		m_hBtnCompareNext->Size
		(
			BTN_COMPARENEXT_X,
			BTN_COMPARENEXT_Y,
			BTN_COMPARENEXT_W(m_Width),
			BTN_COMPARENEXT_H
		);
		//отображение кнопки выбора шрифта
		m_hBtnFont->Size
		(
			BTN_FONT_X,
			BTN_FONT_Y,
			BTN_FONT_W(m_Width),
			BTN_FONT_H
		);

		m_countOfLines = 0;
		return 0;
	}

	//Обработка сообщения WM_COMMAND
	LRESULT OnClick(LPARAM lParam, WPARAM wParam) {

		//Обработка нажатия кнопок ">>" :
		for (int i = 0; i < NUM_OF_FILES; i++) {
			if (lParam == (LPARAM)arrayOfFiles[i].fileOpen.m_btnChoose->m_hWnd) {
				OFN(NULL, arrayOfFiles[i].fileOpen.m_fileName->m_hWnd);
				/*wchar_t buffer[256];
				wsprintfW(buffer, L"%d", n);
				MessageBox(NULL, buffer, NULL, 0);*/
			}
		}

		//Обработка нажатия кнопки "Сравнить":
		if (lParam == (LPARAM)m_hBtnCompare->m_hWnd) {
			for (int i = 0; i < NUM_OF_FILES; i++) {
				arrayOfFiles[i].fileOpen.getFilePath();
			}
			if (m_text.PrepareBufferToPaint(arrayOfFiles, NUM_OF_FILES))
			{			
				for (int i = 0; i < NUM_OF_FILES; i++) {
					arrayOfFiles[i].fileOpen.m_scrollbar->top();
				}

				m_countOfLines = 0;

				long long tempPos = CPainter::GetMaxScrollPos(arrayOfFiles, NUM_OF_FILES);
				tempPos = tempPos - MAX_COUNT_OF_LINES(m_Height) + 2;
				
				
				for (int i = 0; i < NUM_OF_FILES; i++) {
					arrayOfFiles[i].fileOpen.m_scrollbar->setRange(0, tempPos);
				}
				
				m_text.m_toDraw = true;
				InvalidateRect(m_hWnd, NULL, true);
				SetFocus(m_hWnd);
			}
		}

		//Обработка нажатия кнопки "Предыдущее сравнение":
		if (lParam == (LPARAM)m_hBtnComparePrevious->m_hWnd) {

		}

		//Обработка нажатия кнопки "Следующее сравнение":
		if (lParam == (LPARAM)m_hBtnCompareNext->m_hWnd) {

		}

		//Обработка нажатия кнопки "Шрифт":
		if (lParam == (LPARAM)m_hBtnFont->m_hWnd) {
			
			static LOGFONT logfont;
			cf.lStructSize = sizeof(CHOOSEFONT);
			cf.hwndOwner = m_hWnd;
			cf.hDC = NULL;
			cf.lpLogFont = &logfont;
			cf.iPointSize = 0;
			cf.Flags = CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS
				| CF_EFFECTS;
			cf.rgbColors = 0L;
			cf.lCustData = 0L;
			cf.lpfnHook = NULL;
			cf.lpTemplateName = NULL;
			cf.hInstance = NULL;
			cf.lpszStyle = NULL;
			cf.nFontType = 0; // Returned from ChooseFont
			cf.nSizeMin = 0;
			cf.nSizeMax = 0;

			ChooseFont(&cf);
		}
		return 0;
	}

	//Обработка сообщения WM_MOVE
	LRESULT OnMove(LPARAM lParam, WPARAM wParam) { // установление рабочей области окна
		m_countOfLines = 0;
		m_text.m_toDraw = true;
		return 0;
	}

	//Обработка сообщения WM_GETMINMAXINFO
	LRESULT OnGetMinMax(LPARAM lParam, WPARAM wParam) { // изменение позиции окна
		MINMAXINFO* mmi;
		mmi = (MINMAXINFO*)lParam;
		mmi->ptMinTrackSize.x = MIN_PIXELS_TO_DRAW * NUM_OF_FILES;
		mmi->ptMinTrackSize.y = WIN_HEIGHT / 2;
		return 0;
	}

	//Обработка сообщения WM_DESTROY
	LRESULT OnDestroy(LPARAM lParam, WPARAM wParam) { // закрытие окна
		CPainter::CloseHandles(arrayOfFiles, NUM_OF_FILES);
		PostQuitMessage(0);
		return 0;
	}
	#pragma endregion

};
/*
int APIENTRY wWinMain(_In_ HINSTANCE hInstance,
	_In_opt_ HINSTANCE hPrevInstance,
	_In_ LPWSTR    lpCmdLine,
	_In_ int       nCmdShow)
	*/
int main()
{
	// кнопка и надпись "Сравнить файлы (по содержимому)"
	MyWindow* wnd = new MyWindow;
	//wnd->m_hInstance = hInstance;
	
	if (wnd->RegisterWndClass()) {
		if (wnd->Create 
				( // создание и присвоение координат
					0L, 
					L"Window",
					L"Сравнить файлы (по содержимому)",
					WS_OVERLAPPEDWINDOW | WS_VISIBLE,
					0,
					300, 
					WIN_WIDTH,
					WIN_HEIGHT, 
					NULL, 
					0, 
					GetModuleHandle(0),
					wnd
				)
			)
	
		{
			MSG msg;
			while (GetMessage(&msg, nullptr, 0, 0))
			{
				if (msg.hwnd == NULL)
					continue;

				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			return (int)msg.wParam;
		}
		else {
			showError();
			return 0;
		}
	}
	else {
		showError();
		return 0;
	}
}